'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function () {
  return {
    name: 'react-intl-auto',
    visitor: {
      CallExpression(path, state) {
        if (!isValidate(path, state)) {
          return;
        }

        const messagesObj = path.get('arguments')[0];

        let properties;

        if (messagesObj.isObjectExpression()) {
          properties = messagesObj.get('properties');
        } else if (messagesObj.isIdentifier()) {
          const name = messagesObj.node.name;
          const obj = messagesObj.scope.getBinding(name);
          if (!obj) {
            return;
          }
          properties = obj.path.get('init').get('properties');
        }

        if (properties) {
          replaceProperties(properties, state);
        }
      }
    }
  };
};

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const PKG_NAME = 'react-intl';
const FUNC_NAME = 'defineMessages';

const isImportLocalName = (name, { file }) => {
  const imports = file.metadata.modules.imports;
  const intlImports = imports.find(x => x.source === PKG_NAME);
  if (intlImports) {
    const specifier = intlImports.specifiers.find(x => x.imported === FUNC_NAME);
    if (specifier) {
      return specifier.local === name;
    }
  }

  return false;
};

const getPrefix = ({
  file: { opts: { filename } },
  opts: { removePrefix = '', filebase = false }
}) => {
  const file = _path2.default.relative(process.cwd(), filename);
  const fomatted = filebase ? file.replace(/\..+$/, '') : _path2.default.dirname(file);
  const result = fomatted.replace(new RegExp(`^${removePrefix}/?`), '');
  return result;
};

const getId = (path, prefix) => {
  let name;

  if (path.isStringLiteral()) {
    name = path.node.value;
  } else if (path.isIdentifier()) {
    name = path.node.name;
  }

  if (!name) {
    throw new Error(`requires Object key or string literal`);
  }

  return _path2.default.join(prefix, name).replace(/\//g, '.');
};

const isLiteral = node => t.isStringLiteral(node) || t.isTemplateLiteral(node);

const isValidate = (path, state) => {
  const callee = path.get('callee');
  if (!callee.isIdentifier()) {
    return false;
  }

  if (!isImportLocalName(callee.node.name, state)) {
    return false;
  }

  const messagesObj = path.get('arguments')[0];
  if (!messagesObj) {
    return false;
  }

  if (!(messagesObj.isObjectExpression() || messagesObj.isIdentifier())) {
    return false;
  }

  return true;
};

const replaceProperties = (properties, state) => {
  const prefix = getPrefix(state);

  for (const prop of properties) {
    const v = prop.get('value');

    // { defaultMessage: 'hello', description: 'this is hello' }
    if (v.isObjectExpression()) {
      const objProps = v.get('properties');

      // { id: 'already has id', defaultMessage: 'hello' }
      const isNotHaveId = objProps.every(v => v.get('key').node.name !== 'id');
      if (!isNotHaveId) {
        continue;
      }

      const id = getId(prop.get('key'), prefix);

      v.replaceWith(t.objectExpression([t.objectProperty(t.stringLiteral('id'), t.stringLiteral(id)), ...objProps.map(v => v.node)]));

      // 'hello' or `hello ${user}`
    } else if (isLiteral(v)) {
      const id = getId(prop.get('key'), prefix);

      v.replaceWith(t.objectExpression([t.objectProperty(t.stringLiteral('id'), t.stringLiteral(id)), t.objectProperty(t.stringLiteral('defaultMessage'), v.node)]));
    }
  }
};